import nabg


def test_nab_ionize_returns_correctly():
    result = nabg.ionize()
    assert type(result) == str, "ionize() didn't return text"
    assert len(nabg.ionize()) != 0, "ionize() returned nothing"



import re

import patterns
import vocabulary


def test_vocabulary_contains_all_types_used_in_patterns():
    for sentences in patterns.sentence_patterns.values():
        for sentence in sentences:
            for matchobj in re.finditer(r"\$\{([^\}]*)\}", sentence):
                print(matchobj)
                vocab_type = matchobj.group(1)
                assert (
                    vocab_type in vocabulary.bullshit_words
                    and len(vocabulary.bullshit_words[vocab_type]) > 0
                ), f"Could not find words in the vocabulary for type {vocab_type}"



# ---------------------------------------------------------------------------- #
#                                  Vocabulary                                  #
# ---------------------------------------------------------------------------- #

from typing import Dict, List


bullshit_words: Dict[str, List[str]] = {
    "nCosmos": [
        "cosmos",
        "quantum soup",
        "infinite",
        "universe",
        "galaxy",
        "multiverse",
        "grid",
        "quantum matrix",
        "totality",
        "quantum cycle",
        "nexus",
        "planet",
        "solar system",
        "world",
        "stratosphere",
        "dreamscape",
        "biosphere",
        "dreamtime",
    ],
    "nPerson": [
        "being",
        "child",
        "traveller",
        "entity",
        "lifeform",
        "wanderer",
        "visitor",
        "prophet",
        "seeker",
        "Indigo Child",
    ],
    "nPersonPlural": [
        "beings",
        "travellers",
        "entities",
        "lifeforms",
        "dreamweavers",
        "adventurers",
        "pilgrims",
        "warriors",
        "messengers",
        "dreamers",
        "storytellers",
        "seekers",
        "spiritual brothers and sisters",
        "mystics",
        "starseeds",
    ],
    "nMass": [
        "consciousness",
        "nature",
        "beauty",
        "knowledge",
        "truth",
        "life",
        "healing",
        "potential",
        "freedom",
        "purpose",
        "coherence",
        "choice",
        "passion",
        "understanding",
        "balance",
        "growth",
        "inspiration",
        "conscious living",
        "energy",
        "health",
        "spacetime",
        "learning",
        "being",
        "wisdom",
        "stardust",
        "sharing",
        "science",
        "curiosity",
        "hope",
        "wonder",
        "faith",
        "fulfillment",
        "peace",
        "rebirth",
        "self-actualization",
        "presence",
        "power",
        "will",
        "flow",
        "potentiality",
        "chi",
        "intuition",
        "synchronicity",
        "wellbeing",
        "joy",
        "love",
        "karma",
        "life-force",
        "awareness",
        "guidance",
        "transformation",
        "grace",
        "divinity",
        "non-locality",
        "inseparability",
        "interconnectedness",
        "transcendence",
        "empathy",
        "insight",
        "rejuvenation",
        "ecstasy",
        "aspiration",
        "complexity",
        "serenity",
        "intention",
        "gratitude",
        "starfire",
        "manna",
    ],
    "nMassBad": [
        "turbulence",
        "pain",
        "suffering",
        "stagnation",
        "desire",
        "bondage",
        "greed",
        "selfishness",
        "ego",
        "dogma",
        "illusion",
        "delusion",
        "yearning",
        "discontinuity",
        "materialism",
    ],
    "nOurPlural": [
        "souls",
        "lives",
        "dreams",
        "hopes",
        "bodies",
        "hearts",
        "brains",
        "third eyes",
        "essences",
        "chakras",
        "auras",
    ],
    "nPath": [
        "circuit",
        "mission",
        "journey",
        "path",
        "quest",
        "vision quest",
        "story",
        "myth",
    ],
    "nOf": [
        "quantum leap",
        "evolution",
        "spark",
        "lightning bolt",
        "reintegration",
        "vector",
        "rebirth",
        "revolution",
        "wellspring",
        "fount",
        "source",
        "fusion",
        "canopy",
        "flow",
        "network",
        "current",
        "transmission",
        "oasis",
        "quantum shift",
        "paradigm shift",
        "metamorphosis",
        "harmonizing",
        "reimagining",
        "rekindling",
        "unifying",
        "osmosis",
        "vision",
        "uprising",
        "explosion",
    ],
    "ing": [
        "flowering",
        "unfolding",
        "blossoming",
        "awakening",
        "deepening",
        "refining",
        "maturing",
        "evolving",
        "summoning",
        "unveiling",
        "redefining",
        "condensing",
        "ennobling",
        "invocation",
    ],
    "adj": [
        "enlightened",
        "zero-point",
        "quantum",
        "high-frequency",
        "Vedic",
        "non-dual",
        "conscious",
        "sentient",
        "sacred",
        "infinite",
        "primordial",
        "ancient",
        "powerful",
        "spiritual",
        "higher",
        "advanced",
        "internal",
        "sublime",
        "technological",
        "dynamic",
        "life-affirming",
        "sensual",
        "unrestricted",
        "ever-present",
        "endless",
        "ethereal",
        "astral",
        "cosmic",
        "spatial",
        "transformative",
        "unified",
        "non-local",
        "mystical",
        "divine",
        "self-aware",
        "magical",
        "amazing",
        "interstellar",
        "unlimited",
        "authentic",
        "angelic",
        "karmic",
        "psychic",
        "pranic",
        "consciousness-expanding",
        "perennial",
        "heroic",
        "archetypal",
        "mythic",
        "intergalatic",
        "holistic",
        "joyous",
        "eternal",
    ],
    "adjBig": [
        "epic",
        "unimaginable",
        "colossal",
        "unfathomable",
        "magnificent",
        "enormous",
        "jaw-dropping",
        "ecstatic",
        "powerful",
        "untold",
        "astonishing",
        "incredible",
        "breathtaking",
        "staggering",
    ],
    "adjWith": [
        "aglow with",
        "buzzing with",
        "beaming with",
        "full of",
        "overflowing with",
        "radiating",
        "bursting with",
        "electrified with",
    ],
    "adjPrefix": ["ultra-", "supra-", "hyper-", "pseudo-"],
    "vtMass": [
        "inspire",
        "integrate",
        "ignite",
        "discover",
        "rediscover",
        "foster",
        "release",
        "manifest",
        "harmonize",
        "engender",
        "bring forth",
        "bring about",
        "create",
        "spark",
        "reveal",
        "generate",
        "leverage",
    ],
    "vtPerson": [
        "enlighten",
        "inspire",
        "empower",
        "unify",
        "strengthen",
        "recreate",
        "fulfill",
        "change",
        "develop",
        "heal",
        "awaken",
        "synergize",
        "ground",
        "bless",
        "beckon",
    ],
    "viPerson": [
        "exist",
        "believe",
        "grow",
        "live",
        "dream",
        "reflect",
        "heal",
        "vibrate",
        "self-actualize",
    ],
    "vtDestroy": [
        "destroy",
        "eliminate",
        "shatter",
        "disrupt",
        "sabotage",
        "exterminate",
        "obliterate",
        "eradicate",
        "extinguish",
        "erase",
        "confront",
    ],
    "nTheXOf": [
        "richness",
        "truth",
        "growth",
        "nature",
        "healing",
        "knowledge",
        "birth",
        "deeper meaning",
    ],
    "ppPerson": [
        "awakened",
        "re-energized",
        "recreated",
        "reborn",
        "guided",
        "aligned",
    ],
    "ppThingPrep": [
        "enveloped in",
        "transformed into",
        "nurtured by",
        "opened by",
        "immersed in",
        "engulfed in",
        "baptized in",
    ],
    "fixedAdvP": [
        "through non-local interactions",
        "inherent in nature",
        "at the quantum level",
        "at the speed of light",
        "of unfathomable proportions",
        "on a cosmic scale",
        "devoid of self",
        "of the creative act",
    ],
    "fixedAdvPPlace": [
        "in this dimension",
        "outside time",
        "within the Godhead",
        "at home in the cosmos",
    ],
    "fixedNP": [
        "expanding wave functions",
        "superpositions of possibilities",
        "electromagnetic forces",
        "electromagnetic resonance",
        "molecular structures",
        "atomic ionization",
        "electrical impulses",
        "a resonance cascade",
        "bio-electricity",
        "ultrasonic energy",
        "sonar energy",
        "vibrations",
        "frequencies",
        "four-dimensional superstructures",
        "ultra-sentient particles",
        "sub-atomic particles",
        "chaos-driven reactions",
        "supercharged electrons",
        "supercharged waveforms",
        "pulses",
        "transmissions",
        "morphogenetic fields",
        "bio-feedback",
        "meridians",
        "morphic resonance",
        "psionic wave oscillations",
    ],
    "nSubject": [
        "alternative medicine",
        "astrology",
        "tarot",
        "crystal healing",
        "the akashic record",
        "feng shui",
        "acupuncture",
        "homeopathy",
        "aromatherapy",
        "ayurvedic medicine",
        "faith healing",
        "prayer",
        "astral projection",
        "Kabala",
        "reiki",
        "naturopathy",
        "numerology",
        "affirmations",
        "the Law of Attraction",
        "tantra",
        "breathwork",
    ],
    "vOpenUp": [
        "give us access to",
        "enable us to access",
        "remove the barriers to",
        "clear a path toward",
        "let us access",
        "align us with",
        "amplify our connection to",
        "become our stepping-stone to",
        "be a gateway to",
    ],
    "vTraverse": [
        "traverse",
        "walk",
        "follow",
        "engage with",
        "go along",
        "roam",
        "navigate",
        "wander",
        "embark on",
    ],
    "nameOfGod": ["Gaia", "Shiva", "Parvati", "the Goddess", "Shakti"],
    "nBenefits": [
        "improved focus",
        "improved concentration",
        "extreme performance",
        "enlightenment",
        "cellular regeneration",
        "an enhanced sexual drive",
        "improved hydration",
        "psychic rejuvenation",
        "a healthier relationship with the Self",
    ],
    "adjProduct": [
        "alkaline",
        "quantum",
        "holographic",
        "zero-point energy",
        '"living"',
        "metaholistic",
        "ultraviolet",
        "ozonized",
        "ion-charged",
        "hexagonal-cell",
        "organic",
    ],
    "nProduct": [
        "water",
        "healing crystals",
        "Tibetan singing bowls",
        "malachite runes",
        "meditation bracelets",
        "healing wands",
        "rose quartz",
        "karma bracelets",
        "henna tattoos",
        "hemp garments",
        "hemp seeds",
        "tofu",
        "massage oil",
        "herbal incense",
        "cheesecloth tunics",
    ],
}


#! /usr/bin/env python3

"""
Generate randomized bullshhit sentences from a given vocabulary. Default vocabulary and sentence structure
is based on Seb Pearce's "New Age Bullshit Generator". Sentence patterns and vocabulary are defined in the
two accompanying Python files.

This program is derived from Kevin McQuiggin's python2-based CLI of the "New Age Bullshit Generator" by
Seb Pearce:
    New Age Bullshit Generator. Copyright 2014-15 Seb Pearce (sebpearce.com).
    Conversion by Kevin McQuiggin. Copyright 2017 K. McQuiggin.

Rewritten for python3 with additional features by Naveen Unnikrishnan.
Copyright Naveen Unnikrishnan, January 2021.
Licensed under the MIT License.
"""
__version__ = "1.0.0"

import random
import re
from typing import Dict, List

import click

import patterns
import vocabulary


class BullshitGenerator:
    """
    BullshitGenerator class. Feed it sentence patterns and associated vocabulary
    to randomly generate bullshit sentences.

    Attributes:
        sentence_pool (Dict[str, List[str]]): The complete corpus of sentence patterns separated into topics.
        sentence_patterns (Dict[str, List[str]]): The remaining sentence patterns yet to be used in a run.
        vocabulary (Dict[str, List[str]]): The vocabulary of terms separated into types.
    """

    sentence_pool: Dict[str, List[str]]
    sentence_patterns: Dict[str, List[str]]
    vocabulary: Dict[str, List[str]]

    def __init__(
        self, sentence_patterns: Dict[str, List[str]], vocabulary: Dict[str, List[str]]
    ):
        """
        Constructor for BullshitGenerator.

        Args:
            sentence_patterns (Dict[str, List[str]]): The corpus of sentence patterns separated into topics.
            vocabulary (Dict[str, List[str]]): The vocabulary of terms separated into types.
        """
        self.sentence_pool = sentence_patterns
        self.sentence_patterns = sentence_patterns
        self.vocabulary = vocabulary
        self.shuffle_sentence_patterns()

    # ---------------------------------------------------------------------------- #
    #                               Utility functions                              #
    # ---------------------------------------------------------------------------- #

    @staticmethod
    def replace_a_with_an(sentence: str) -> str:
        """
        Replace 'a [vowel]' with 'an [vowel]'.

        Args:
            sentence (str): Sentence to modify

        Returns:
            str: Sentence with 'a [vowel]' replaced with 'an [vowel]'
        """
        p = re.compile(r"(^|\W)([Aa]) ([aeiou])")
        return p.sub(r"\1\2n \3", sentence)

    @staticmethod
    def insert_space_between_sentences(text: str) -> str:
        """
        Insert a space after periods and question marks.

        Args:
            text (str): Paragraph to be formatted

        Returns:
            str: Formatted paragraph.
        """
        p = re.compile(r"([\.\?])(\w)")
        return p.sub(r"\1 \2", text)

    @staticmethod
    def clean_sentence(sentence: str) -> str:
        """
        Tidy up a generated sentence.

        Args:
            sentence (str): Generated sentence to clean up

        Returns:
            str: Cleaned-up sentence
        """
        result = BullshitGenerator.replace_a_with_an(sentence)
        result = result[0].upper() + result[1:]
        return result

    # ---------------------------------------------------------------------------- #
    #                 Support routines related to the main program                 #
    # ---------------------------------------------------------------------------- #

    def shuffle_sentence_patterns(self):
        """
        Shuffle sentence patterns.
        """
        for sentenceList in self.sentence_patterns.values():
            random.shuffle(sentenceList)

    def reset_sentence_patterns(self):
        """
        Reset sentence patterns for a new run.
        """
        self.sentence_patterns = self.sentence_pool
        self.shuffle_sentence_patterns()

    def get_random_topic(self) -> str:
        """
        Choose a topic at random from the sentence pool.

        Returns:
            str: Randomly chosen topic
        """
        return random.choice(list(self.sentence_patterns.keys()))

    def retrieve_random_word_of_type(self, type: str) -> str:
        """
        Choose a random vocabulary word, based on a given word type.

        Args:
            type (str): Type of vocabulary word

        Returns:
            str: A vocabulary word of the requested type
        """
        return random.choice(self.vocabulary[type])

    def replace_vocab_patterns(self, sentence: str) -> str:
        """
        Replace type placeholders in the pattern with random words from the vocabulary.

        Args:
            sentence (str): Sentence to be modified

        Returns:
            str: Sentence where type placeholders have been replaced with random words from the vocabulary
        """
        sentence = re.sub(
            r"\$\{([^\}]*)\}",
            lambda matchobj: self.retrieve_random_word_of_type(matchobj.group(1)),
            sentence,
        )
        return sentence

    def generate_sentence(self, topic: str) -> str:
        """
        Generate a single sentence on a particular topic.

        Args:
            topic (str): Topic on which to generate a sentence

        Raises:
            KeyError: If topic is invalid

        Returns:
            str: Generated sentence
        """
        try:
            sentences = self.sentence_patterns[topic]
            if len(sentences) == 0:
                raise KeyError
        except KeyError:
            raise KeyError("Invalid topic")
        pattern = sentences.pop()
        result = self.replace_vocab_patterns(pattern)
        if len(sentences) == 0:
            self.sentence_patterns.pop(topic, None)
        result = self.clean_sentence(result)
        return result

    def generate_text(self, number_of_sentences: int, sentence_topic: str) -> str:
        """
        Generate a set of sentences.

        Args:
            number_of_sentences (int): Number of sentences to generate
            sentence_topic (str): Topic to generate sentences in

        Returns:
            str: Generated text
        """
        full_text = ""
        for _ in range(number_of_sentences):
            if sentence_topic not in self.sentence_patterns:
                sentence_topic = random.choice(list(self.sentence_patterns.keys()))
            full_text += self.generate_sentence(sentence_topic)
        full_text = self.insert_space_between_sentences(full_text)
        return full_text

    # ---------------------------------------------------------------------------- #
    #                                 Main program                                 #
    # ---------------------------------------------------------------------------- #

    def ionize(self, number_of_sentences: int = 1, topic: str = None) -> str:
        """
        Generate bullshit.

        Args:
            number_of_sentences (int, optional): Number of sentences to be generated. Defaults to 1.
            topic (str, optional): Topic on which to generate text. Picks one at random if not provided.

        Returns:
            str: Generated bullshit.
        """
        if topic is None:
            topic = self.get_random_topic()
        return self.generate_text(number_of_sentences, topic)


# ---------------------------------------------------------------------------- #
#                                     NABG                                     #
# ---------------------------------------------------------------------------- #


def ionize(number_of_sentences: int = 1, topic: str = None):
    """
    Generate new-age bullshit.

    Args:
        number_of_sentences (int, optional): Number of sentences to generate. Defaults to 1.
        topic (str, optional): Topic on which to generate text. Picks one at random if not provided.
    """
    bullshit_generator = BullshitGenerator(
        patterns.sentence_patterns, vocabulary.bullshit_words
    )
    return bullshit_generator.ionize(number_of_sentences, topic)


@click.command(context_settings=dict(help_option_names=["-h", "--help"]))
@click.option("-n", default=1, help="Number of sentences to generate.")
@click.option(
    "--topic", "-t", default=None, help="Topic on which to generate bullshit."
)
@click.option(
    "--list-topics", "-l", is_flag=True, default=False, help="List available topics."
)
def main(n: int, topic: str, list_topics: bool):
    """
    Generate new-age bullshit.
    """
    if list_topics:
        for topic in patterns.sentence_patterns.keys():
            print(topic)
        return
    print(ionize(n, topic))



# ---------------------------------------------------------------------------- #
#                               Sentence patterns                              #
# ---------------------------------------------------------------------------- #

from typing import Dict, List


sentence_patterns: Dict[str, List[str]] = {
    "explain": [
        "${nMass} is the driver of ${nMass}.",
        "${nMass} is the ${nTheXOf} of ${nMass}, and of us.",
        "You and I are ${nPersonPlural} of the ${nCosmos}.",
        "We exist as ${fixedNP}.",
        "We ${viPerson}, we ${viPerson}, we are reborn.",
        "Nothing is impossible.",
        "This life is nothing short of a ${ing} ${nOf} of ${adj} ${nMass}.",
        'Consciousness consists of ${fixedNP} of quantum energy. "Quantum" means a ${ing} of the ${adj}.',
        "The goal of ${fixedNP} is to plant the seeds of ${nMass} rather than ${nMassBad}.",
        "${nMass} is a constant.",
        "By ${ing}, we ${viPerson}.",
        "The ${nCosmos} is ${adjWith} ${fixedNP}.",
        "To ${vTraverse} the ${nPath} is to become one with it.",
        "Today, science tells us that the essence of nature is ${nMass}.",
        "${nMass} requires exploration.",
    ],
    "warn": [
        "We can no longer afford to live with ${nMassBad}.",
        "Without ${nMass}, one cannot ${viPerson}.",
        "Only a ${nPerson} of the ${nCosmos} may ${vtMass} this ${nOf} of ${nMass}.",
        "You must take a stand against ${nMassBad}.",
        "Yes, it is possible to ${vtDestroy} the things that can ${vtDestroy} us, but not without ${nMass} on our side.",
        "${nMassBad} is the antithesis of ${nMass}.",
        "You may be ruled by ${nMassBad} without realizing it. Do not let it ${vtDestroy} the ${nTheXOf} of your ${nPath}.",
        "The complexity of the present time seems to demand a ${ing} of our ${nOurPlural} if we are going to survive.",
        "${nMassBad} is born in the gap where ${nMass} has been excluded.",
        "Where there is ${nMassBad}, ${nMass} cannot thrive.",
    ],
    "hope": [
        "Soon there will be a ${ing} of ${nMass} the likes of which the ${nCosmos} has never seen.",
        "It is time to take ${nMass} to the next level.",
        "Imagine a ${ing} of what could be.",
        "Eons from now, we ${nPersonPlural} will ${viPerson} like never before as we are ${ppPerson} by the ${nCosmos}.",
        "It is a sign of things to come.",
        "The future will be a ${adj} ${ing} of ${nMass}.",
        "This ${nPath} never ends.",
        "We must learn how to lead ${adj} lives in the face of ${nMassBad}.",
        "We must ${vtPerson} ourselves and ${vtPerson} others.",
        "The ${nOf} of ${nMass} is now happening worldwide.",
        "We are being called to explore the ${nCosmos} itself as an interface between ${nMass} and ${nMass}.",
        "It is in ${ing} that we are ${ppPerson}.",
        "The ${nCosmos} is approaching a tipping point.",
        "${nameOfGod} will ${vOpenUp} ${adj} ${nMass}.",
    ],
    "yourProblems": [
        "Although you may not realize it, you are ${adj}.",
        "${nPerson}, look within and ${vtPerson} yourself.",
        "Have you found your ${nPath}?",
        "How should you navigate this ${adj} ${nCosmos}?",
        "It can be difficult to know where to begin.",
        "If you have never experienced this ${nOf} ${fixedAdvP}, it can be difficult to ${viPerson}.",
        "The ${nCosmos} is calling to you via ${fixedNP}. Can you hear it?",
    ],
    "history": [
        "Throughout history, humans have been interacting with the ${nCosmos} via ${fixedNP}.",
        "Reality has always been ${adjWith} ${nPersonPlural} whose ${nOurPlural} are ${ppThingPrep} ${nMass}.",
        "Our conversations with other ${nPersonPlural} have led to a ${ing} of ${adjPrefix}${adj} consciousness.",
        "Humankind has nothing to lose.",
        "We are in the midst of a ${adj} ${ing} of ${nMass} that will ${vOpenUp} the ${nCosmos} itself.",
        "Who are we? Where on the great ${nPath} will we be ${ppPerson}?",
        "We are at a crossroads of ${nMass} and ${nMassBad}.",
    ],
    "selling": [
        "Through ${nSubject}, our ${nOurPlural} are ${ppThingPrep} ${nMass}.",
        "${nSubject} may be the solution to what's holding you back from a ${adjBig} ${nOf} of ${nMass}.",
        "You will soon be ${ppPerson} by a power deep within yourself - a power that is ${adj}, ${adj}.",
        "As you ${viPerson}, you will enter into infinite ${nMass} that transcends understanding.",
        "This is the vision behind our 100% ${adjProduct}, ${adjProduct} ${nProduct}.",
        "With our ${adjProduct} ${nProduct}, ${nBenefits} is only the beginning.",
    ],
}


from setuptools import setup

with open("README.md", "r") as fh:
    long_description = fh.read()

setup(
    name="nabg",
    version="0.0.1",
    description="A customizable bullshit generator based on Seb Pearce's 'New-Age Bullshit Generator'",
    py_modules=["nabg", "patterns", "vocabulary"],
    package_dir={"": "src"},
    classifiers=[
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Typing :: Typed",
    ],
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=["click > 7.0"],
    extras_require={"dev": ["pytest>=6.2.2"]},
    url="https://github.com/naveen-u/nabg",
    author="Naveen Unnikrishnan",
    author_email="naveenunnikrishnan98@gmail.com",
    entry_points={
        "console_scripts": [
            "nabg=nabg:main",
        ]
    },
)
